package gs.smolban.model

import java.time.Instant
import cats.Show

/**
  * Describes an instant at which something was created. Opaque type for 
  * `java.time.Instant`.
  */
opaque type CreatedAt = Instant

object CreatedAt:

  /**
   * Instantiate a new [[CreatedAt]].
   *
   * @param timestamp The underlying timestamp.
   * @return The new instance.
   */
  def apply(timestamp: Instant): CreatedAt = timestamp

  given CanEqual[CreatedAt, CreatedAt] = CanEqual.derived

  given Show[CreatedAt] = _.toInstant().toString()

  extension (createdAt: CreatedAt)
    /**
     * Unwrap this value.
     *
     * @return The underlying `Instant` value.
     */
    def toInstant(): Instant = createdAt

end CreatedAt
