package gs.smolban.model

import cats.Show

case class Ticket(
  id: Long,
  group: Group.Id,
  createdAt: CreatedAt
)

object Ticket:

  /**
   * Unique identifier - relative to some [[Group]] - for a [[Ticket]]. This is 
   * an opaque type for a Long. In general, [[Ticket]] identifiers are sequences 
   * within a group.
   */
  opaque type Id = Long

  object Id:

    /**
     * Instantiate a new [[Ticket.Id]].
     *
     * @param id The underlying Long.
     * @return The new [[Ticket.Id]] instance.
     */
    def apply(id: Long): Id = id

    given CanEqual[Id, Id] = CanEqual.derived

    given Show[Id] = _.toLong().toString()

    extension (id: Id)
      /**
       * Unwrap this Ticket ID.
       *
       * @return The underlying Long value.
       */
      def toLong(): Long = id

  end Id

end Ticket
